@@ -28,24 +28,24 @@
 
 #pragma once
 
-#include "cif++/Structure.hpp"
+#include <cif++.hpp>
 
 namespace pdb_redo
 {
 
-using mmcif::Atom;
-using mmcif::Point;
-
 // --------------------------------------------------------------------
 // Class used in calculating radii
 
 class AtomShape
 {
   public:
-	AtomShape(const Atom &atom, float resHigh, float resLow,
-		bool electronScattering);
-	AtomShape(const Atom &atom, float resHigh, float resLow,
-		bool electronScattering, float bFactor);
+	AtomShape(cif::row_handle atom, cif::row_handle atom_aniso, float resHigh, float resLow,
+		bool electronScattering, std::optional<float> bFactor = {});
+
+	AtomShape(const cif::mm::atom &atom, float resHigh, float resLow, bool electronScattering, std::optional<float> bFactor = {})
+		: AtomShape(atom.get_row(), atom.get_row_aniso(), resHigh, resLow, electronScattering, bFactor)
+	{
+	}
 
 	~AtomShape();
 
@@ -54,7 +54,7 @@ class AtomShape
 
 	float radius() const;
 	float calculatedDensity(float r) const;
-	float calculatedDensity(Point p) const;
+	float calculatedDensity(cif::point p) const;
 
   private:
 	struct AtomShapeImpl *mImpl;
